from django.db.models.signals import pre_save
from django.dispatch import receiver
from arkindex.dataimport.models import WorkerRun
from rest_framework.exceptions import ValidationError


@receiver(pre_save, sender=WorkerRun)
def check_parents(sender, instance, **kwargs):
    """
    Since we can't properly validate parents field on WorkerRun by simply defining an Array of UUIDs,
    we add a pre-save signal with constraints.
    It will check that provided UUIDs refer to existing WorkerRun objects and that these objects are
    linked to the same DataImport as the to-be-saved WorkerRun instance.
    """
    parents = WorkerRun.objects.filter(id__in=instance.parents, dataimport=instance.dataimport).exclude(id=instance.id).values_list('id', flat=True)
    if set(parents) != set(instance.parents):
        raise ValidationError(f"Can't add or update WorkerRun {instance.id} because parents field isn't properly defined. It can be either because"
                              " one or several UUIDs don't refer to existing WorkerRuns or either because listed WorkerRuns doesn't belong to the"
                              " same DataImport than this WorkerRun.")
