from pathlib import Path
from unittest.mock import patch, call
from django.core.management import call_command
from django.core.management.base import CommandError
from arkindex.project.tests import FixtureTestCase

TEI_SAMPLES = Path(__file__).resolve().parent.parent / 'tei_samples'


@patch('arkindex.himanis.management.commands.import_tei.TeiParser')
class TestImportTeiCommand(FixtureTestCase):

    def test_run(self, parser_mock):
        call_command(
            'import_tei',
            TEI_SAMPLES,
            corpus=self.corpus,
        )
        self.assertEqual(parser_mock.call_count, 3)
        self.assertCountEqual(parser_mock.call_args_list, [
            call(
                TEI_SAMPLES / filename,
                self.corpus,
                self.corpus.types.get(slug='volume'),
                self.corpus.types.get(slug='act'),
            )
            for filename in ('arguments.xml', 'update_after.xml', 'update_before.xml')
        ])
        self.assertEqual(parser_mock().run.call_count, 3)

    def test_path_not_found(self, parser_mock):
        with self.assertRaisesRegex(CommandError, 'does not exist'):
            call_command(
                'import_tei',
                Path('/nowhere'),
                corpus=self.corpus,
            )

    def test_wrong_folder_type(self, parser_mock):
        with self.assertRaises(
                CommandError,
                msg="Folder type with slug 'burrito' in corpus Unit Tests does not exist"):
            call_command(
                'import_tei',
                TEI_SAMPLES,
                corpus=self.corpus,
                folder_slug='burrito',
            )

    def test_wrong_act_type(self, parser_mock):
        with self.assertRaises(
                CommandError,
                msg="Act type with slug 'act' in corpus Unit Tests does not exist"):
            call_command(
                'import_tei',
                TEI_SAMPLES,
                corpus=self.corpus,
                act_slug='burrito',
            )
