from arkindex.project.tests import FixtureAPITestCase
from io import StringIO
from django.core.management import call_command
from django.urls import reverse
from rest_framework import status


class TestSchemaView(FixtureAPITestCase):

    def test_constant_schema(self):
        """
        Ensure that what a SchemaView outputs is always exactly the same
        as what manage.py generateschema outputs.

        Note that we use the JSON output format instead of the normal YAML because JSON will not allow
        Python-specific objects, while PyYAML would use YAML tags such as "!!python/object",
        and it is much faster to render.
        """
        stdout = StringIO()
        call_command(
            'generateschema',
            generator_class='arkindex.project.openapi.SchemaGenerator',
            format='openapi-json',
            stdout=stdout,
        )
        expected_schema = stdout.getvalue().strip()

        def _test_schema():
            response = self.client.get(reverse('api:openapi-schema') + '?format=openapi-json')
            self.assertEqual(response.status_code, status.HTTP_200_OK)
            self.assertEqual(response.content.decode('utf-8'), expected_schema)

        _test_schema()

        self.client.force_login(self.user)
        _test_schema()

        self.client.force_login(self.superuser)
        _test_schema()
